@@ -4,9 +4,6 @@ import json
 from time import sleep
 from typing import List, Dict, Any
 from datetime import datetime, timedelta
-from os.path import exists, join
-from os import makedirs
-from appdirs import site_data_dir
 from redzoo.database.simple import SimpleDB
 from homeconnect_webthing.auth import Auth
 from homeconnect_webthing.eventstream import EventListener
@@ -20,6 +17,16 @@ class OfflineException(Exception):
 
 class Appliance(EventListener):
 
+    ON = "On"
+    OFF = "Off"
+
+    IDLING = "IDLING"
+    STARTABLE = "STARTABLE"
+    DELAYED_STARTED = "DELAYED_STARTED"
+    RUNNING = "RUNNING"
+    FINISHED = "FINISHED"
+    VALID_STATES = [IDLING, STARTABLE, DELAYED_STARTED, RUNNING, FINISHED]
+
     def __init__(self, device_uri: str, auth: Auth, name: str, device_type: str, haid: str, brand: str, vib: str, enumber: str):
         self._device_uri = device_uri
         self._auth = auth
@@ -32,8 +39,8 @@ class Appliance(EventListener):
         self.__value_changed_listeners = set()
         self.last_refresh = datetime.now() - timedelta(hours=9)
         self.remote_start_allowed = False
-        self.startable = False
-        self.started = False
+        self.startable = False   # deprecated
+        self.started = False      # deprecated
         self.program_remote_control_active = False
         self._program_selected = ""
         self.program_remaining_time_sec = 0
@@ -42,10 +49,9 @@ class Appliance(EventListener):
         self._power = ""
         self._door = ""
         self._operation = ""
-        self.completed = True
-        self.status = "INACTIVE"
         self.__program_active = ""
         self.child_lock = False
+        self.__db = SimpleDB(haid + '_db')
         self._reload_status_and_settings()
         self._reload_selected_program(ignore_error=True)
 
@@ -57,7 +63,7 @@ class Appliance(EventListener):
         if len(self._power) > 0:
             return self._power[self._power.rindex('.')+1:]
         else:
-            return "Off"
+            return self.OFF
 
     @property
     def door(self):
@@ -68,7 +74,7 @@ class Appliance(EventListener):
 
     @property
     def operation(self):
-        if len(self._operation)> 0:
+        if len(self._operation) > 0:
             return self._operation[self._operation.rindex('.')+1:]
         else:
             return ""
@@ -91,41 +97,54 @@ class Appliance(EventListener):
         self.__value_changed_listeners.add(value_changed_listener)
         self._notify_listeners()
 
-    def __update_state(self):
-        if self.power.lower() == "on" and self.operation.lower() == 'delayedstart':
-            new_status = "DELAYED_STARTED"
-
-        elif self.power.lower() == "on" and self.operation.lower() == 'run':
-            new_status = "RUNNING"
-            self.completed = False
-
-        elif self.power.lower() == "on" and \
-             not self.completed and \
-             self.door.lower() == "closed" and \
-             self.remote_start_allowed and \
-             self.operation.lower() not in ['delayedstart', 'run', 'finished', 'inactive']:
-            new_status = "REMOTE_STARTABLE"
-
-        elif self.power.lower() != "on":
-            new_status = "INACTIVE"
-            self.completed = True
-
-        else:
-            new_status = "INACTIVE"
+    @property
+    def status(self) -> str:
+        return self.__db.get("state", self.IDLING)
 
+    @status.setter
+    def status(self, new_status: str):
         if self.status != new_status:
             logging.info("new status: " + new_status + " (previous: " + self.status + ")")
-            self.status = new_status
+            self.__db.put("state", new_status)
 
+    @property
+    def __previous_run_completed(self) -> bool:
+        return self.__db.get("previous_program_completed", True)
+
+    @__previous_run_completed.setter
+    def __previous_run_completed(self, completed: bool):
+        self.__db.put("previous_program_completed", completed)
+
+    def __update_state(self):
+        # power off (completes the program)
+        if self.power.lower() != self.ON.lower():
+            self.status = self.IDLING
+            self.__previous_run_completed = True
+        # power on
+        else:
+            if self.operation.lower() == 'delayedstart':
+                self.status = self.DELAYED_STARTED
+            elif self.operation.lower() == 'run':
+                self.status = self.RUNNING
+                self.__previous_run_completed = False
+            elif self.operation.lower() == 'ready' and self.door.lower() == "closed" and self.__previous_run_completed and self.remote_start_allowed:
+                self.status = self.STARTABLE
+            elif self.operation.lower() == 'finished':
+                self.status = self.FINISHED
+            else:
+                self.status = self.IDLING
+                if self.door.lower() == 'open':
+                    self.__previous_run_completed = True
 
     def _notify_listeners(self):
         self.__update_state()
 
-        # will be removed
+        #  # deprecated
         self.startable = self.door.lower() == "closed" and \
                          self.power.lower() == "on" and \
                          self.remote_start_allowed and \
                          self.operation.lower() not in ['delayedstart', 'run', 'finished', 'inactive']
+        # deprecated
         self.started = self.power.lower() == "on" and \
                        self.door.lower() == "closed" and \
                        self.operation.lower() in ['delayedstart', 'run']
@@ -141,7 +160,12 @@ class Appliance(EventListener):
         logging.info(self.name + " has been disconnected (event stream)")
 
     def on_keep_alive_event(self, event):
-        self._notify_listeners()
+        try:
+            if (self.last_refresh + timedelta(minutes=30)) < datetime.now():
+                self._reload_status_and_settings()
+            self._notify_listeners()
+        except Exception as e:
+            logging.warning("error occurred processing keep alive event "+  str(e))
 
     def on_notify_event(self, event):
         self._on_value_changed_event(event)
@@ -170,6 +194,7 @@ class Appliance(EventListener):
             self._on_values_changed(settings, "reload settings")
         except Exception as e:
             if isinstance(e, OfflineException):
+                self._power = ""
                 logging.info(self.name + " is offline. Could not query current status/settings")
             else:
                 logging.warning(self.name + " error occurred on refreshing" + str(e))
@@ -410,7 +435,7 @@ class FinishInAppliance(Appliance):
         self._program_finish_in_relative_sec = 0
         self.__program_finish_in_relative_max_sec = 86000
         self.__program_finish_in_relative_stepsize_sec = 60
-        self._durations = SimpleDB(haid + '_durations', sync_period_sec=0)
+        self._durations = SimpleDB(haid + '_durations')
         super().__init__(device_uri, auth, name, device_type, haid, brand, vib, enumber)
 
     def _on_value_changed(self, key: str, change: Dict[str, Any], source: str) -> bool:
