@@ -124,9 +124,9 @@ class ApplianceThing(Thing):
                      'status',
                      self.status,
                      metadata={
-                         'title': 'status',
+                         'title': 'Status',
                          "type": "string",
-                         'description': 'The status',
+                         'description': 'The status (valid values ' + ", ".join(appliance.VALID_STATES) + ')',
                          'readOnly': True,
                      }))
 
